@@ -7,7 +7,7 @@
 #ifndef BOOST_CONTEXT_FIBER_H
 #define BOOST_CONTEXT_FIBER_H
 
-#include <boost/predef.h>
+#include <boost/predef/os.h>
 #if BOOST_OS_MACOS
 #define _XOPEN_SOURCE 600
 #endif
@@ -16,6 +16,7 @@ extern "C" {
 #include <ucontext.h>
 }
 
+#include <boost/predef.h>
 #include <boost/context/detail/config.hpp>
 
 #include <algorithm>
@@ -65,12 +66,19 @@ namespace detail {
 // tampoline function
 // entered if the execution context
 // is resumed for the first time
-template< typename Record >
-static void fiber_entry_func( void * data) noexcept {
-    Record * record = static_cast< Record * >( data);
-    BOOST_ASSERT( nullptr != record);
-    // start execution of toplevel context-function
-    record->run();
+template <typename Record>
+#if BOOST_OS_MACOS
+static void fiber_entry_func(std::uint32_t data_high,
+                             std::uint32_t data_low) noexcept {
+  auto data =
+      reinterpret_cast<void *>(std::uint64_t(data_high) << 32 | data_low);
+#else
+static void fiber_entry_func(void *data) noexcept {
+#endif
+  Record *record = static_cast<Record *>(data);
+  BOOST_ASSERT(nullptr != record);
+  // start execution of toplevel context-function
+  record->run();
 }
 
 struct BOOST_CONTEXT_DECL fiber_activation_record {
@@ -112,7 +120,7 @@ struct BOOST_CONTEXT_DECL fiber_activation_record {
     fiber_activation_record( stack_context sctx_) noexcept :
         sctx( sctx_ ),
         main_ctx( false ) {
-    } 
+    }
 
     virtual ~fiber_activation_record() {
 #if defined(BOOST_USE_TSAN)
@@ -281,7 +289,7 @@ public:
             c = boost::context::detail::invoke( fn_, std::move( c) );
 #else
             c = std::invoke( fn_, std::move( c) );
-#endif  
+#endif
         } catch ( forced_unwind const& ex) {
             c = Ctx{ ex.from };
         }
@@ -304,7 +312,7 @@ static fiber_activation_record * create_fiber1( StackAlloc && salloc, Fn && fn)
     void * storage = reinterpret_cast< void * >(
             ( reinterpret_cast< uintptr_t >( sctx.sp) - static_cast< uintptr_t >( sizeof( capture_t) ) )
             & ~ static_cast< uintptr_t >( 0xff) );
-    // placment new for control structure on context stack
+    // placement new for control structure on context stack
     capture_t * record = new ( storage) capture_t{
             sctx, std::forward< StackAlloc >( salloc), std::forward< Fn >( fn) };
     // stack bottom
@@ -328,7 +336,15 @@ static fiber_activation_record * create_fiber1( StackAlloc && salloc, Fn && fn)
     record->uctx.uc_stack.ss_size = reinterpret_cast< uintptr_t >( storage) -
             reinterpret_cast< uintptr_t >( stack_bottom) - static_cast< uintptr_t >( 64);
     record->uctx.uc_link = nullptr;
-    ::makecontext( & record->uctx, ( void (*)() ) & fiber_entry_func< capture_t >, 1, record);
+#if BOOST_OS_MACOS
+    const auto integer = std::uint64_t(record);
+    ::makecontext(&record->uctx, (void (*)()) & fiber_entry_func<capture_t>, 2,
+                  std::uint32_t((integer >> 32) & 0xFFFFFFFF),
+                  std::uint32_t(integer));
+#else
+    ::makecontext(&record->uctx, (void (*)()) & fiber_entry_func<capture_t>, 1,
+                  record);
+#endif
 #if defined(BOOST_USE_ASAN)
     record->stack_bottom = record->uctx.uc_stack.ss_sp;
     record->stack_size = record->uctx.uc_stack.ss_size;
@@ -341,13 +357,13 @@ static fiber_activation_record * create_fiber1( StackAlloc && salloc, Fn && fn)
 
 template< typename Ctx, typename StackAlloc, typename Fn >
 static fiber_activation_record * create_fiber2( preallocated palloc, StackAlloc && salloc, Fn && fn) {
-    typedef fiber_capture_record< Ctx, StackAlloc, Fn >  capture_t; 
+    typedef fiber_capture_record< Ctx, StackAlloc, Fn >  capture_t;
 
     // reserve space for control structure
     void * storage = reinterpret_cast< void * >(
             ( reinterpret_cast< uintptr_t >( palloc.sp) - static_cast< uintptr_t >( sizeof( capture_t) ) )
             & ~ static_cast< uintptr_t >( 0xff) );
-    // placment new for control structure on context stack
+    // placement new for control structure on context stack
     capture_t * record = new ( storage) capture_t{
             palloc.sctx, std::forward< StackAlloc >( salloc), std::forward< Fn >( fn) };
     // stack bottom
@@ -371,7 +387,15 @@ static fiber_activation_record * create_fiber2( preallocated palloc, StackAlloc
     record->uctx.uc_stack.ss_size = reinterpret_cast< uintptr_t >( storage) -
             reinterpret_cast< uintptr_t >( stack_bottom) - static_cast< uintptr_t >( 64);
     record->uctx.uc_link = nullptr;
-    ::makecontext( & record->uctx,  ( void (*)() ) & fiber_entry_func< capture_t >, 1, record);
+#if BOOST_OS_MACOS
+    const auto integer = std::uint64_t(record);
+    ::makecontext(&record->uctx, (void (*)()) & fiber_entry_func<capture_t>, 2,
+                  std::uint32_t((integer >> 32) & 0xFFFFFFFF),
+                  std::uint32_t(integer));
+#else
+    ::makecontext(&record->uctx, (void (*)()) & fiber_entry_func<capture_t>, 1,
+                  record);
+#endif
 #if defined(BOOST_USE_ASAN)
     record->stack_bottom = record->uctx.uc_stack.ss_sp;
     record->stack_size = record->uctx.uc_stack.ss_size;
@@ -397,14 +421,6 @@ private:
 	template< typename Ctx, typename StackAlloc, typename Fn >
 	friend detail::fiber_activation_record * detail::create_fiber2( preallocated, StackAlloc &&, Fn &&);
 
-    template< typename StackAlloc, typename Fn >
-    friend fiber
-    callcc( std::allocator_arg_t, StackAlloc &&, Fn &&);
-
-    template< typename StackAlloc, typename Fn >
-    friend fiber
-    callcc( std::allocator_arg_t, preallocated, StackAlloc &&, Fn &&);
-
     detail::fiber_activation_record   *   ptr_{ nullptr };
 
     fiber( detail::fiber_activation_record * ptr) noexcept :
